# Generated by Django 3.1.2 on 2020-10-18 16:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0046_auto_20201018_1743'),
    ]

    operations = [
        migrations.AlterField(
            model_name='email',
            name='typ',
            field=models.CharField(blank=True, choices=[('', '-'), ('kunde_registriert', 'Registrierungsmail Kunde'), ('bestellung_rechnung', 'Rechnung'), ('bestellung_stock_warning', 'Warnung zum Lagerbestand')], default='', max_length=50, verbose_name='Typ'),
        ),
    ]
