# Generated by Django 3.2 on 2021-04-12 14:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0056_auto_20210409_2340'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='email',
            options={'default_permissions': ('add', 'change', 'view', 'delete', 'send'), 'verbose_name': 'E-Mail', 'verbose_name_plural': 'E-Mails'},
        ),
        migrations.AddField(
            model_name='email',
            name='sent',
            field=models.BooleanField(default=False, verbose_name='Gesendet?'),
        ),
    ]
