# Generated by Django 4.1.5 on 2023-02-03 18:12

import django.core.validators
from django.db import migrations, models
import kmuhelper.modules.main.models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0087_bestellung_bezahlt_am_bestellung_versendet_am'),
    ]

    operations = [
        migrations.AlterField(
            model_name='bestellung',
            name='bezahlt',
            field=models.BooleanField(default=False, help_text='Mehr Infos -> <a href="https://rafaelurben.github.io/django-kmuhelper/manual/faq#was-passiert-wenn-ich-eine-bestellung-als-bezahltversendet-markiere" target="_blank">FAQ</a>', verbose_name='Bezahlt'),
        ),
        migrations.AlterField(
            model_name='bestellung',
            name='versendet',
            field=models.BooleanField(default=False, help_text='Mehr Infos -> <a href="https://rafaelurben.github.io/django-kmuhelper/manual/faq#was-passiert-wenn-ich-eine-bestellung-als-bezahltversendet-markiere" target="_blank">FAQ</a>', verbose_name='Versendet'),
        ),
        migrations.AlterField(
            model_name='bestellung',
            name='zahlungskonditionen',
            field=models.CharField(default=kmuhelper.modules.main.models.defaultzahlungskonditionen, help_text='Skonto und Zahlungsfrist -> <a href="https://rafaelurben.github.io/django-kmuhelper/manual/faq#wie-funktionieren-zahlungskonditionen" target="_blank">Mehr erfahren</a>', max_length=16, validators=[django.core.validators.RegexValidator('^([0-9]+(\\.[0-9]+)?:[0-9]+;)*0:[0-9]+$', "Bitte benutze folgendes Format: 'p:d;p:d' - p = Skonto in %; d = Tage")], verbose_name='Zahlungskonditionen'),
        ),
    ]
