# Generated by Django 3.0.4 on 2020-07-16 13:25

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0020_auto_20200716_1443'),
    ]

    operations = [
        migrations.CreateModel(
            name='versandliste',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('kmuhelper.bestellung',),
        ),
    ]
