# Generated by Django 3.2 on 2021-04-20 19:46

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0070_auto_20210417_2146'),
    ]

    operations = [
        migrations.CreateModel(
            name='PaymentImport',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('time_imported', models.DateTimeField(auto_now_add=True, verbose_name='Importiert am')),
                ('is_parsed', models.BooleanField(default=False, verbose_name='Verarbeitet?')),
                ('data_msgid', models.CharField(max_length=50, verbose_name='Nachrichtenid')),
                ('data_creationdate', models.DateTimeField(verbose_name='Erstellt am')),
            ],
            options={
                'verbose_name': 'Zahlungsimport',
                'verbose_name_plural': 'Zahlungsimporte',
            },
        ),
        migrations.CreateModel(
            name='PaymentImportEntry',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('ref', models.CharField(max_length=50, verbose_name='Referenznummer')),
                ('iban', models.CharField(max_length=22, verbose_name='IBAN')),
                ('name', models.CharField(max_length=100, verbose_name='Name')),
                ('amount', models.FloatField(verbose_name='Betrag')),
                ('currency', models.CharField(max_length=10, verbose_name='Währung')),
                ('parent', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='entries', to='kmuhelper.paymentimport')),
            ],
            options={
                'verbose_name': 'Zahlungsimport-Einträge',
                'default_permissions': (),
            },
        ),
    ]
