# Generated by Django 3.1 on 2020-09-22 19:49

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0033_auto_20200802_2105'),
    ]

    operations = [
        migrations.AddField(
            model_name='bestellungsposten',
            name='rabatt',
            field=models.IntegerField(default=0, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(100)], verbose_name='Rabatt in %'),
        ),
    ]
