# Generated by Django 3.0.4 on 2020-04-26 16:20

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0002_auto_20200426_1746'),
    ]

    operations = [
        migrations.AlterField(
            model_name='kosten',
            name='name',
            field=models.CharField(default='Zusätzliche Kosten', max_length=500, verbose_name='Name'),
        ),
    ]
