<div class="pagechooser {{ pagechooserclass }}">
    {% for group in griddata %}
        <div class="split">
            {% for elem in group %}
                <a href="{{ elem.url }}" class="{{ elem.class }}" target="_top">
                    <div class="center">
                        <i class="fa-2x fa-fw {{ elem.icon }}"></i>
                        <br><br>
                        <h2>{{ elem.title }}</h2>
                        {% if elem.subtitle %}
                        <p>{{ elem.subtitle }}</p>
                        {% endif %}
                    </div>
                </a>
            {% endfor %}
        </div>
        <br>
    {% endfor %}
</div>
