{% extends 'admin/kmuhelper/_special/app/change_list.html' %}

{% block modulename %}Zahlungseingang{% endblock %}

{% block content %}
    <p id="pageinfo">
        Hier befinden sich Bestellungen, welche noch nicht als bezahlt markiert wurden.
        <br>
        Hinweis: Es können auch <a target="_blank" href="{% url 'admin:kmuhelper_paymentimport_upload' %}">camt.053d&nbsp;XML-Dateien</a> importiert werden.
    </p>
    <br>

{{ block.super }}
{% endblock content %}
