# Generated by Django 3.0.4 on 2020-08-01 22:18

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('kmuhelper', '0031_auto_20200727_1603'),
    ]

    operations = [
        migrations.AlterField(
            model_name='bestellung',
            name='datum',
            field=models.DateTimeField(default=django.utils.timezone.now, verbose_name='Datum'),
        ),
    ]
